<?php
/*
 * timeline-db.php
 * Description: Database functions for the SIMILE Timline Plugin
 * Plugin URI: http://www.freshlabs.de/journal/archives/2006/10/wordpress-plugin-simile-timeline/
 * Author: Tim Isenheim
 * 
	===========================================================================
	SIMILE Timeline for WordPress
	Copyright (C) 2006 Tim Isenheim
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	===========================================================================
*/

/* ---------------------------------------------------------------------------------
 * adds an event end date for posts
 * --------------------------------------------------------------------------------*/
function stl_timeline_eventend_addcolumn() {
	global $wpdb;
	$column_name = "stl_timeline_event_end";
	if(!stl_timeline_column_exists($column_name)) {
		$wpdb->query("ALTER TABLE $wpdb->posts ADD COLUMN " . $column_name . " datetime NOT NULL DEFAULT '0000-00-00 00:00:00'");
	}
}

/* ---------------------------------------------------------------------------------
 * removes the database column on uninstalling
 * WARNING: This deletes all end dates set for posts
 * --------------------------------------------------------------------------------*/
function stl_timeline_eventend_removecolumn() {
	global $wpdb;
	if(stl_timeline_column_exists("stl_timeline_event_end")) {
		$wpdb->query("ALTER TABLE $wpdb->posts DROP COLUMN stl_timeline_event_end");
	}
}

/* ---------------------------------------------------------------------------------
 * Save the end date of an event(post) to the database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_eventend_box($pID){
	global $wpdb;
	
	if (!empty ($_POST['edit_timeline_end_date'])) {
		$stl_aa = $_POST['stl_aa'];
		$stl_mm = $_POST['stl_mm'];
		$stl_jj = $_POST['stl_jj'];
		$stl_hh = $_POST['stl_hh'];
		$stl_mn = $_POST['stl_mn'];
		$stl_ss = $_POST['stl_ss'];
		$stl_jj = ($stl_jj > 31) ? 31 : $stl_jj;
		$stl_hh = ($stl_hh > 23) ? $stl_hh -24 : $stl_hh;
		$stl_mn = ($stl_mn > 59) ? $stl_mn -60 : $stl_mn;
		$stl_ss = ($stl_ss > 59) ? $stl_ss -60 : $stl_ss;
		$_POST['stl_timeline_event_end'] = "$stl_aa-$stl_mm-$stl_jj $stl_hh:$stl_mn:$stl_ss";
		
		$stl_tee = $_POST['stl_timeline_event_end'];

		//extract($_POST);
	
		$wpdb->query("UPDATE $wpdb->posts SET	stl_timeline_event_end = '$stl_tee' WHERE ID = $pID");
	}
}

/* ---------------------------------------------------------------------------------
 * stl_get_extreme_blog_date
 * Get the date of the very first or last blog post
 * --------------------------------------------------------------------------------*/
function stl_get_extreme_blog_date($minmax){
	global $wpdb;
	$query = "SELECT $minmax(id) from $wpdb->posts";
	$post_id = $wpdb->get_var($query);
	$date_query = "SELECT post_date_gmt from $wpdb->posts WHERE id=$post_id";
	
	$extreme_blog_date = mysql2date('M d Y H:i:s', $wpdb->get_var($date_query));
	return $extreme_blog_date . " GMT" . date('O');
}

/* ---------------------------------------------------------------------------------
 * install STL table for Timeline category attributes
 * --------------------------------------------------------------------------------*/
function stl_timeline_add_relationship_table(){
		
	global $wpdb;

	$table_name = $wpdb->prefix . "stl_timeline_terms";
	if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {

		$sql = "CREATE TABLE " . $table_name . " (
			  id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
			  term_id INT NOT NULL,
			  color VARCHAR( 7 ) NOT NULL DEFAULT '#58A0DC',
			  active INT NOT NULL DEFAULT 0
			);";

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		dbDelta($sql);
		
		// copy all category-terms to the new table
		$sql2 = "SELECT * FROM $wpdb->term_taxonomy WHERE taxonomy = 'category' OR taxonomy = 'link_category'";

		$res = $wpdb->get_results($sql2);

		foreach($res as $term){
			stl_add_term_to_db($term);
		}
   }
}

/* ---------------------------------------------------------------------------------
 * update STL table for Timeline categories
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_relationship_table(){
	
	global $wpdb;
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		
	// copy diff of term table to timeline-terms
	$table = $wpdb->prefix . "stl_timeline_terms";
	$sql = "SELECT term_id FROM $table ";
	$res = $wpdb->get_results($sql, ARRAY_A);
	
	$terms = get_terms(array('category', 'link_category'));
	
	$cmp = array();
	// copy categories to stl_timeline term relationship table
	foreach($res as $stl_term){
		array_push($cmp, $stl_term['term_id']);
	}
	// loop all available valid terms
	foreach($terms as $term){		
		// add to database if not already exist in the timeline terms 
		if(!in_array($term->term_id, $cmp)){
			stl_add_term_to_db($term);
		}
		else{
			// already exists.. do nothing
		}
	}
}

/* ---------------------------------------------------------------------------------
 * remove category from options if the whole category is deleted
 * --------------------------------------------------------------------------------*/
function stl_delete_cat_from_options($cat_ID) {
	$s = trim(get_option('stl_timelinecategories'));
	$s = str_replace(",$cat_ID", "", $s);
	update_option('stl_timelinecategories', $s);
}

/* ---------------------------------------------------------------------------------
 * remove term from options if the whole category is deleted
 * --------------------------------------------------------------------------------*/
function stl_delete_term_from_db($term_id) {
	global $wpdb;

	$table_name = $wpdb->prefix . "stl_timeline_terms";
	
	$insert = "DELETE FROM " . $table_name . " WHERE term_id= " . $term_id . "";
	$results = $wpdb->query($insert);
}

/* ---------------------------------------------------------------------------------
 * add single term to timeline relationship table
 * --------------------------------------------------------------------------------*/
function stl_add_term_to_db($term) {
	global $wpdb;
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";

	$insert = "INSERT INTO " . $table_name . " (term_id) " .
		  	  "VALUES ('".$term->term_id."')";
	$results = $wpdb->query($insert);

}

/* ---------------------------------------------------------------------------------
 * get term from timeline relationship table for specific post ID
 * --------------------------------------------------------------------------------*/
function stl_get_timeline_term_for_post($post_id) {
	global $wpdb;
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";
	
	$term_id = $wpdb->get_var("SELECT term_id FROM $wpdb->term_taxonomy " .
			"INNER JOIN $wpdb->term_relationships ON($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id AND
			$wpdb->term_relationships.object_id = $post_id)");

	return stl_get_term($term_id);
}

/* ---------------------------------------------------------------------------------
 * get term from timeline relationship table for specific post ID
 * --------------------------------------------------------------------------------*/
function stl_get_term($term_id) {
	global $wpdb;
	
	$table_name = $wpdb->prefix . "stl_timeline_terms";

	$get = "SELECT * FROM " . $table_name . " WHERE term_id = $term_id";
	$row = $wpdb->get_row($get);
	return $row;
}

/* ---------------------------------------------------------------------------------
 * Save the category active attribute into database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_catactive($cID, $status){
	global $wpdb;
	//extract($_POST);
	$wpdb->query("UPDATE ".$wpdb->prefix."stl_timeline_terms SET active = '$status' WHERE term_id = $cID");
}

/* ---------------------------------------------------------------------------------
 * Save the category color into database
 * --------------------------------------------------------------------------------*/
function stl_timeline_update_catcolor($cID, $color){
	global $wpdb;
	//extract($_POST);
	$wpdb->query("UPDATE ".$wpdb->prefix."stl_timeline_terms SET color = '$color' WHERE term_id = $cID");
}

/* ---------------------------------------------------------------------------------
 * Get functions for the Event end date
 * --------------------------------------------------------------------------------*/
function stl_get_end_time( $d = '' ) {
	if ( '' == $d )
		$the_time = stl_get_eventend_time(get_settings('time_format'));
	else
		$the_time = stl_get_eventend_time($d);
	return apply_filters('stl_get_eventend_time', $the_time, $d);
}

/* ---------------------------------------------------------------------------------
 * returns timestamp
 * --------------------------------------------------------------------------------*/
function stl_get_eventend_time( $d = 'U' ) { 
	global $post;

	$time = $post->stl_timeline_event_end;

	$time = mysql2date($d, $time);
	return apply_filters('stl_get_eventend_time', $time, $d);
}

/* ---------------------------------------------------------------------------------
 * checks if a given column exists in a database table
 * --------------------------------------------------------------------------------*/
function stl_timeline_column_exists($col_name){
	global $wpdb;
	$column_exists = false;
	$q = $wpdb->query("SHOW COLUMNS FROM $wpdb->posts LIKE '$col_name'");
	if($q == 1){
		$column_exists = true;
	}
	return $column_exists;
}

/* ---------------------------------------------------------------------------------
     Uninstall WordPress user option in database named $option
 * --------------------------------------------------------------------------------*/
function stl_uninstall_option($option){
	if(get_option($option))
		delete_option($option);
}

?>